use serde::{Deserialize, Serialize};

#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
#[non_exhaustive]
pub struct RepositoryVulnerabilityAlertWebhookEventPayload {
    pub action: RepositoryVulnerabilityAlertWebhookEventAction,
    pub enterprise: Option<serde_json::Value>,
    pub alert: serde_json::Value,
}

#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
#[serde(rename_all = "snake_case")]
#[non_exhaustive]
pub enum RepositoryVulnerabilityAlertWebhookEventAction {
    Create,
    Dismiss,
    Reopen,
    Resolve,
}
